﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace hcXNA
{
    // box entity
    public class hcBox
    {
        // members
        private string _id;
        private Vector2 _stringSize;
        private Texture2D _tex;
        private Vector2 _origin;
        private Vector2 _pos, _targetPos;
        private float _speed;
        private float _size, _targetSize;
        private float _alpha, _targetAlpha;
        private Color _color;

        // ctor 1 :: with centered text
        public hcBox(int id, Vector2 pos, Texture2D tex, SpriteFont font, float speed, float size, float alpha)
        {
            _id = id.ToString();
            _stringSize = font.MeasureString(_id) / 2.0f;
            _tex = tex;
            _origin = new Vector2(_tex.Width / 2, _tex.Height / 2);
            _pos = _targetPos = pos;
            _size = _targetSize = size;
            _alpha = _targetAlpha = alpha;
            _color = Color.White * _alpha;
            _speed = speed;
        }

        // ctor 2 :: without Text
        public hcBox(Vector2 pos, Texture2D tex, float speed, float size, float alpha)
        {
            _id = "";
            _stringSize = new Vector2();
            _tex = tex;
            _origin = new Vector2(_tex.Width / 2, _tex.Height / 2);
            _pos = _targetPos = pos;
            _size = _targetSize = size;
            _alpha = _targetAlpha = alpha;
            _color = Color.White * _alpha;
            _speed = speed;
        }

        // update() :: linear interpolation
        public void update()
        {
            _pos += (_targetPos - _pos) * _speed;
            _size += (_targetSize - _size) * _speed;
            _alpha += (_targetAlpha - _alpha) * _speed;
            _color = Color.White * _alpha;
        }

        // setPosition() ::  set target position
        public void setPosition(Vector2 targetPosition)
        {
            _targetPos = targetPosition;
        }

        // setDeltaX() :: set target position along X axis
        public void setDeltaX(float dx)
        {
            _targetPos.X += dx;
        }

        // setDeltaY() :: set target position along Y axis
        public void setDeltaY(float dy)
        {
            _targetPos.Y += dy;
        }

        // setAlpha() :: set target alpha
        public void setAlpha(float targetAlpha)
        {
            _targetAlpha = targetAlpha;
        }

        // getAlpha() :: get alpha
        public float getAlpha()
        {
           return _alpha;
        }

        // forceAlpha() :: force alpha value, immediate
        public void forceAlpha(float alpha)
        {
            _targetAlpha = _alpha = alpha;
        }

        // setSpeed()
        public void setSpeed(float speed)
        {
            _speed = speed;
        }

        // setSize() :: set target size
        public void setSize(float targetSize)
        {
            _targetSize = targetSize;
        }
        
        // setParams() :: set parameters at once (immediate)
        public void setParams(Vector2 targetPosition, float targetAlpha, float targetSize)
        {
            _targetPos = _pos = targetPosition;
            _targetAlpha = _alpha = targetAlpha;
            _targetSize = _size = targetSize;
        }

        // draw() :: takes spritebatch to draw
        public void draw(SpriteBatch spriteBatch, SpriteFont font)
        {
            spriteBatch.Draw(_tex, _pos, null, _color, 0.0f, _origin, _size, SpriteEffects.None, 0.0f);
            spriteBatch.DrawString(font, _id, _pos - _stringSize, Color.Black);
        }

        // draw() :: takes spritebatch to draw without text
        public void draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(_tex, _pos, null, _color, 0.0f, _origin, _size, SpriteEffects.None, 0.0f);
        }

        // drawDisplay() :: draw displayed version
        public void drawDisplay(SpriteBatch spriteBatch, SpriteFont font, Vector2 pos)
        {
            spriteBatch.Draw(_tex, pos, null, _color, 0.0f, _origin, _size * 4.0f, SpriteEffects.None, 0.0f);
            spriteBatch.DrawString(font, _id, pos - _stringSize, Color.Black);
        }

        // drawSimpleColor
        public void drawSimpleColor(SpriteBatch spriteBatch, Color color)
        {
            spriteBatch.Draw(_tex, _pos, null, color, 0.0f, _origin, _size, SpriteEffects.None, 0.0f);
        }
    }

    // =========================================================================
    // box handler
    public class hcBoxHandler
    {
        private Vector2 _stripPos;
        private hcBox _stripBg;
        private Texture2D _stripTex;

        private int _numItems = 50;
        private List<hcBox> _items;
        private Texture2D _itemTex;
        private SpriteFont _font;
        private int _currentItem;

        private float _gridSize, _gridOffsetX;
        private Vector2 _displayPos;

        private List<hcBox> _displayQueue;

        private hcBox _userIcon;
        private Vector2 _userIconPos;
        private Texture2D _userTex;
        private int _userPresence = 1;

        private float _transSpeed;
        private bool _ANIMATED = true;

        // =================================================
        // CTOR
        public hcBoxHandler(ContentManager content, hcSetting setting)
        {
            _font = content.Load<SpriteFont>("CambriaRegular");
            _itemTex = content.Load<Texture2D>("Textures/frame");
            _stripTex = content.Load<Texture2D>("Textures/strip");
            _userTex = content.Load<Texture2D>("Textures/icon_user");

            // strip
            _stripPos = new Vector2(setting.HCenter, setting.Height - 100);
            _stripBg = new hcBox(-1, _stripPos, _stripTex, _font, 1.0f, 3.5f, 0.5f);

            // item positions
            _gridSize = setting.Width / 9.0f;
            _currentItem = _numItems / 2;
            _gridOffsetX = setting.HCenter - (_currentItem * _gridSize);

            _transSpeed = setting.transSpeed;

            // hcBoxes
            _items = new List<hcBox>();
            for (int i = 0; i < _numItems; i++)
            {
                Vector2 itemPosition = new Vector2(_gridOffsetX + _gridSize * i, setting.Height - 100);
                hcBox item = new hcBox(i, itemPosition, _itemTex, _font, _transSpeed, 0.2f, 1.0f);
                _items.Add(item);
            }

            // displayPos
            _displayPos = new Vector2(setting.HCenter, setting.VCenter - 80);
            _displayQueue = new List<hcBox>();

            // add displayQueue an item with 0.001 alph
            hcBox displayItem = new hcBox(_currentItem, _displayPos, _itemTex, _font, _transSpeed, 1.2f, 0.001f);
            displayAdd(displayItem);

            // create user icon
            _userIconPos = new Vector2(setting.Width - 50, 50);
            _userIcon = new hcBox(_userIconPos, _userTex, _transSpeed, 0.5f, 1.0f);
        }

        // toggleTransition()
        public void toggleTransition()
        {
            // toggle animation
            _ANIMATED = !_ANIMATED;
            
            // get nerdy
            if (_ANIMATED)
            {
                foreach (hcBox b in _items) b.setSpeed(_transSpeed);
            }
            else
            {
                foreach (hcBox b in _items) b.setSpeed(1.0f);
            }
        }

        // displayAdd()
        public void displayAdd(hcBox box)
        {
            box.setAlpha(1.0f);
            box.setSize(0.8f);
            _displayQueue.Add(box);
        }

        // swipeLeft(amt)
        public void swipeRight(int amt)
        {
            if (_currentItem == _numItems - 1) return; // boundary check

            // iterate
            for (int i = 0; i < amt; i++)
            {
                if (_currentItem == _numItems - 1) return; // boundary check
                foreach (hcBox b in _items) b.setDeltaX(-_gridSize);
                
                _currentItem++;

                // add displayQueue an item with 0.001 alph
                float tspeed = _transSpeed;
                if (!_ANIMATED) tspeed = 1.0f;
                hcBox item = new hcBox(_currentItem, _displayPos, _itemTex, _font, tspeed, 1.2f, 0.001f);
                displayAdd(item);
            }
        }

        // swipeRight(amt)
        public void swipeLeft(int amt)
        {
            if (_currentItem == 0) return; // boundary check

            // iterate
            for (int i = 0; i < amt; i++)
            {
                if (_currentItem == 0) return; // boundary check

                foreach (hcBox b in _items) b.setDeltaX(_gridSize);
                
                _currentItem--;

                // add displayQueue an item with 0.001 alph
                float tspeed = _transSpeed;
                if (!_ANIMATED) tspeed = 1.0f;
                hcBox item = new hcBox(_currentItem, _displayPos, _itemTex, _font, tspeed, 1.2f, 0.001f);
                displayAdd(item);
            }
        }

        // setUserPresence(int)
        public void setUserPresence(int presence)
        {
            _userPresence = presence;
        }
        
        // update
        public void update(GameTime gameTime)
        {
            // update items
            foreach (hcBox b in _items)
            {
                b.update();
            }

            // update displayQueue
            foreach (hcBox b in _displayQueue)
            {
                b.update();
            }

            // if displayQueue size > 5, delete the items from the head
            if (_displayQueue.Count > 5)
            {
                for (int i = 0; i < _displayQueue.Count - 5; i++)
                {
                    _displayQueue[i].setAlpha(0.0f);
                    _displayQueue[i].setSize(0.5f);
                    if ( _displayQueue[i].getAlpha() < 0.001f ) _displayQueue.RemoveAt(i);
                }
            }
        }

        // draw
        public void draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Begin(SpriteSortMode.Deferred, BlendState.NonPremultiplied);
            {
                // draw strip
                _stripBg.draw(spriteBatch);

                // draw items
                foreach (hcBox b in _items)
                {
                    b.draw(spriteBatch, _font);
                }

                // draw displayed version
                foreach (hcBox b in _displayQueue)
                {
                    b.draw(spriteBatch, _font);
                }

                // if skeleton is detected, draw user icon
                if ( _userPresence == 1 ) _userIcon.drawSimpleColor(spriteBatch, Color.Gray);
                else if (_userPresence == 2) _userIcon.drawSimpleColor(spriteBatch, Color.Green);
            }
            spriteBatch.End();
        }       
    }
}
